      module mod_AddrCas
!
!-----------------------------------------------------------------
! Utilitaires pour l'adressage de fonctions CAS
!-----------------------------------------------------------------
!
      integer,private :: Two_pow(0:31)
      logical :: yprepare=.false.
      contains
!********************************************************************
!********************************************************************
       subroutine prepare_AddrCas
      yprepare=.true.
      do k=0,31
         Two_pow(k)=2**k
      enddo
      end subroutine
!********************************************************************
!********************************************************************
C      integer M(10),N(10),ll(10)
C      NB=4
C      M(1:NB)=(/3,4,2,4/)
C      MT=product(M(1:NB))
C      print'(a,10i4)','M',M
C      print'(a,i4)','MT',MT
C      do L=1,MT
C         call A_inverse2(L,N,M,NB,MT)
C         call A_direct(L1,N,M,NB)
C         print'(a,i4,2x,12i4)','L,N',L,N(1:NB),l1
C      enddo
C! Extraire un segment de bits d'un entier
C      i=895439099
C      ideb=3
C      ifin=11
C      call extr_segm_bits(i,ideb,ifin)
C      end

!********************************************************************
!********************************************************************

!===============================================================
! Fonctions (adresses de fragments) --> (adresse totale)
!           et inverse  
!
! On considere NB fragments, et NB determinants (1 par fragment)
! soit N(1), N(2), ... N(NB) les adresses de ces det dans leur fragment
! soit M(1), M(2), ... M(NB) l'adresse max dans chaque fragment
! ( 1 <= N(i) <= M(i) )
! Soit L l'adresse totale de la reunion des fragments
!
! subroutine A_Fr__Mol : {N(i)} --> L
! subroutine A_Mol__Fr : L --> {N(i)}
!
!===============================================================

       subroutine A_Fr__Mol(L,N,M,NB)
      integer,intent(in) :: N(NB),M(NB),NB
      integer,intent(out) :: L
! L=i+(j-1)*Mi+(k-1)*Mi*Mj+(l-1)*Mi*Mj*Mk+...
      L=N(1)
      do k=2,NB
         L=L+(N(k)-1)*product(M(1:k-1))
      enddo
      end subroutine


!********************************************************************
       subroutine A_Mol__Fr_up(L,N,M,NB)
      integer,intent(in) :: L,M(NB),NB
      integer,intent(out) :: N(NB)
      integer,allocatable :: ll(:)
!--------------------------------------------------
! N est calcule par valeurs croissantes
!  en entree : L,   M,NB,MT
!  en sortie : N
!--------------------------------------------------
      call A_Mol__Fr_err(L,M,NB)
      allocate(ll(nb))
      ll(1)=l
      N(1)=mod(l-1,M(1))+1
      do i=2,NB
         ll(i)=(ll(i-1)-N(i-1))/M(i-1) + 1
         N(i)=mod(ll(i)-1,m(i)) + 1
      enddo
      deallocate(ll)
      end subroutine
!********************************************************************
       subroutine A_Mol__Fr_down(L,N,M,NB)
      integer,intent(in) :: L,M(NB),NB
      integer,intent(out) :: N(NB)
      integer,allocatable :: ll(:)
!--------------------------------------------------
! N calcule par valeurs decroissantes (autre formule)
!  en entree : L,   M,NB,MT
!  en sortie : N
!--------------------------------------------------
      call A_Mol__Fr_err(L,M,NB)
      allocate(ll(nb))
      n(nb)=(l-1)/product(m(1:nb-1)) + 1
      ll(nb)=l
      do i=nb-1,1,-1
         ll(i)=ll(i+1)-(n(i+1)-1)*product(m(1:i))
         n(i)=(ll(i)-1)/product(m(1:i-1)) + 1
      enddo
      !print'(a,i4,2x,3i4,2x,12i4)','LL,N',ll(1:NB),N(1:NB),l
      deallocate(ll)
      end subroutine
!********************************************************************
       subroutine A_Mol__Fr_updown(L,N,M,NB)
      integer,intent(in) :: L,M(NB),NB
      integer,intent(out) :: N(NB)
      integer,allocatable :: ll(:)
!--------------------------------------------------
! L: numero de la liste de la molecule (en donnee)
! N(:): numeros des listes des fragments (en sortie)
! M(:): valeur max de N(:) (en donnee)
! NB: nb de fragments

! N calcule par valeurs croissantes pour les valeurs basses
! N calcule par valeurs decroissantes pour les valeurs hautes
! devrait etre la solution la plus rapide
!  en entree : L,   M,NB,MT
!  en sortie : N
!--------------------------------------------------
      call A_Mol__Fr_err(L,M,NB)
      allocate(ll(nb))
!--------------
      ll(1)=l
      N(1)=mod(l-1,M(1))+1
      do i=2,NB/2
         ll(i)=(ll(i-1)-N(i-1))/M(i-1) + 1
         N(i)=mod(ll(i)-1,m(i)) + 1
      enddo
!--------------
      n(nb)=(l-1)/product(m(1:nb-1)) + 1
      ll(nb)=l
      do i=nb-1,nb/2+1,-1
         ll(i)=ll(i+1)-(n(i+1)-1)*product(m(1:i))
         n(i)=(ll(i)-1)/product(m(1:i-1)) + 1
      enddo
      !print'(a,i4,2x,3i4,2x,12i4)','LL,N',ll(1:NB),N(1:NB),l
      deallocate(ll)
      end subroutine


!*************************************************************
       subroutine A_Mol__Fr_err(L,M,NB)
      integer,intent(in) :: L,M(NB)
      if(L>product(M)) then
         print*,'*****************************************'
         print*,'Erreur : adresse de molecule trop grande'
         print*,' Adresse=',L
         print 1000,' Adresse max des fragments',M
         print*,'Produit des adresses max des fragments',product(M)
         print*,'STOP'
         print*,'*****************************************'
         stop
      endif
 1000 format(a,20i4)
      end subroutine
!*************************************************************
       subroutine extr_segm_bits(i,j,ideb,ifin)
!-------------------------------------------------------------
! Soit un determinant d'une molecule, represente par les bits d'un 
! entier
! la molecule est divisee en fragments
! on veut recuperer les determinants de chaque fragment
!-------------------------------------------------------------
      integer,intent(in) :: i    !det de la molecule
      integer,intent(in) :: ideb,ifin ! delimitations du fragment
      integer,intent(out) :: j     !det du frament
      logical yw
      if(.not.yprepare) then
         print*,'il faut appeler Prepare_Addrcas avant'
         stop
      endif
      yw=.true.
      if(yw) then
         print*,' '
         print*,'*********************************'
         print*,'        extr_segm_bits'
         print*,'*********************************'
         print*,' '
         print*,'Entier en entree',i
         print*,' '
         print*,'representation en bits'
         call write_bits(i)
         print'(32a1)',(' ',k=1,ideb-1),'|',(' ',k=ideb+1,ifin-1),'|'
         print*,' '
         jj=i/2**(ideb)
         print*,'decalage a gauche',ideb
         call write_bits(jj)
         print*,'troncature a droite'
         kk=mod(i,2**ifin)
         call write_bits(kk)
         ll=mod(j,2**ifin)
         call write_bits(ll)
      endif
      j=mod(i/two_pow(ideb-1),two_pow(ifin-1))
         print*,'fragment extrait',ideb,ifin
         call write_bits(j,ideb,ifin)
      end subroutine
!*************************************************************
!*************************************************************
       subroutine write_bits(i,ideb,ifin,advance)
!
!---------------------------------------------------------------------
! i: integer representing the string to be printed
! ideb,ifin: define the range of the printed string (default all bits)
!---------------------------------------------------------------------
!
      intent(in) i
      integer,intent(in),optional :: ideb,ifin
      character(*),optional :: advance
      character(3) :: advance_loc
      logical yc(0:31)
      jdeb=0
      jfin=31
      if(present(ideb)) jdeb=ideb
      if(present(ifin)) jfin=ifin
      do k=0,31
         yc(k)=(i==ibset(i,k))
      enddo
      if(present(advance)) then
         advance_loc=advance
      else
         advance_loc='YES'
      endif
      write(6,'(32l1)',advance=advance_loc) yc(0:jfin-jdeb)
      end subroutine
!*************************************************************
!*************************************************************
      end module
!*************************************************************
!*************************************************************
      
